﻿-- ----------------------------------------------------------------------------
-- GridAutoFrameSize by smuggles
-- ----------------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("GridAutoFrameSize")
local GridL = AceLibrary("AceLocale-2.2"):new("Grid")
local AceOO = AceLibrary("AceOO-2.0")
local RL = AceLibrary("Roster-2.1")

local math_ceil = math.ceil
local math_max = math.max
local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers = GetNumRaidMembers
local GetRaidRosterInfo = GetRaidRosterInfo

GridAutoFrameSize = Grid:NewModule("GridAutoFrameSize")
GridAutoFrameSize.defaultDB = {
	autoSizeFrames = 1
}

function GridAutoFrameSize:OnEnable()
	local opt = GridLayout.options.args
	opt["autoSizeFrames"] = {
		type = "toggle",
		name = L["Auto Size Raid Frame"],
		desc = L["Automatically adjusts frame layout to raid size"],
		order = 29,
		get = function ()
			return self.db.profile.autoSizeFrames
		end,
		set = function (v)
			self.db.profile.autoSizeFrames = v
			self:CheckRoster()
		end,	
	}
	
	self:RegisterBucketEvent({"RAID_ROSTER_UPDATE", "PARTY_MEMBERS_CHANGED"}, 1, "CheckRoster")
end

function GridAutoFrameSize:CheckRoster()
	if (not self.db.profile.autoSizeFrames) then
		return
	end
	
	local newLayout
	local raidSize = GetNumRaidMembers()
	if raidSize > 0 then
		
		local maxGroup = math_ceil(raidSize / 5)
		for i = 1, raidSize do
			local group = select(3, GetRaidRosterInfo(i))
			maxGroup = math_max(maxGroup, group)
		end
		if (maxGroup > 5) or (raidSize > 25) then
			newLayout = "By Group 40"
		else
			newLayout = "By Group " .. 5*maxGroup
		end
		self:Debug(("raid settings: size=%d, maxgroup=%d => layout=%s"):format(raidSize, maxGroup, newLayout))

	elseif GetNumPartyMembers() > 0 then
		newLayout = "By Group 5"
		self:Debug(("in party or not grouped => layout=%s"):format(raidSize, maxGroup, newLayout))
	end
	
	newLayout = newLayout and GridL[newLayout]
	
	if not newLayout or newLayout == GridLayout.db.profile.layout then
		self:Debug("no layout change")		
		return
	end
	
	self:Debug("new layout: "..newLayout)		
	GridLayout:LoadLayout(newLayout)
	
end
